#!/bin/ksh 

#**************************************************************
# Author: Sebastien Denvil, Martial Mancip
# Contact: Martial.Mancip_ipsl.jussieu.fr
# $Date$
# $Author$
# $Revision$
# IPSL (2006)
#  This software is governed by the CeCILL licence see libIGCM/libIGCM_CeCILL.LIC
# History:
# Modification:
#
#**************************************************************

#=========================================================
# The documentation of this file can be automatically generated
# if you use the prefix #D- for comments to be extracted. 
# Extract with command: cat lib* | grep "^#D-" | cut -c "4-"
#=========================================================
 
#D-#==================================================
#D-LibIGCM_sys for claude and other LMD computers
#D-#==================================================
#D-
#D- This ksh library if a layer under some usefull 
#D- environment variables and shell commands.
#D- All those definitions depend on host particularities.
#D- It manages a stack mechanism and test validity of operations.
#D- All function described bellow must be prefixed by IGCM_sys.

#====================================================
# libIGCM_sys PARAMETERS
#====================================================

#====================================================
#set DEBUG_sys to true to output calls of function
typeset -r DEBUG_sys=${DEBUG_sys:=true}

#====================================================
# Turn in dry run mode ? (sys_Put_Rest, sys_Put_Out, sys_Get)
typeset -r DRYRUN=${DRYRUN:=0}

# YOU MUST COMPILE YOUR EXE FILES FOR DRYRUN MODE !
# -------------------------------------------------------------------------------------
# | DRYRUN=  |  Date computations, | sys_Get    |  Exe    | sys_Put_Out; sys_Put_Rest |
# |          |  Cp/Exe param files |            |  Chmod  |                           |
# |          |      Qsub           |            |         |                           |
# -------------------------------------------------------------------------------------
# |    0     |       yes           |    yes     |  yes    |      yes                  |
# -------------------------------------------------------------------------------------
# |    1     |       yes           |    yes     |  yes    |      no                   |
# -------------------------------------------------------------------------------------
# |    2     |       yes           |    yes     |  no     |      no                   |
# -------------------------------------------------------------------------------------
# |    3     |       yes           |    no      |  no     |      no                   |
# -------------------------------------------------------------------------------------

#=====================================================
#Global Variables :
#=====================================================
# Language : "fr" or "en"
typeset -r MYLANG="fr"

#=====================================================
# Host and user names 
# $hostname ou hostname 
typeset -r HOST=$( hostname )
# $username ou whoami
typeset -r LOGIN=$( whoami )

#D-
#D-#==================================================
#D-Program used in libIGCM
#D-#==================================================

# rsync with path
typeset -r RSYNC=/usr/bin/rsync  
#RSYNC_opt args to rsync
typeset -r RSYNC_opt="-Lt -v"

#====================================================
# Source Ferret
#. /home/${LOGIN}/.atlas_env_${HOST}_ksh
export FER_ATLAS /usr/local/atlas
export FER_GO "$FER_GO $FER_ATLAS"
export FER_DATA "$FER_DATA $FER_ATLAS"
export FER_PALETTE "$FER_PALETTE $FER_ATLAS"
export FER_EXTERNAL_FUNCTIONS "$FER_EXTERNAL_FUNCTIONS $FER_ATLAS"
export PATH {$FER_ATLAS}:{$PATH}

#====================================================
# Host specific DIRECTORIES
#====================================================

#====================================================
#- R_EXE   (==> BIN_DIR = ${MODIPSL}/bin )
typeset -r R_EXE="${MODIPSL}/bin"

#====================================================
#- libIGCM_POST
typeset -r libIGCM_POST=${libIGCM}

#====================================================
#- SCRATCHDIR (=> ${R_DONNEES})
typeset -r SCRATCHDIR=${TMPDIR}

#====================================================
#- RUN_DIR_PATH : Temporary working directory (=> TMP)
typeset -r RUN_DIR_PATH=${RUN_DIR_PATH:=${SCRATCHDIR}/tmp$$}

#====================================================
#- SUBMIT_DIR : submission dir
typeset SUBMIT_DIR=${SUBMIT_DIR:=${PWD}}

#====================================================
#- ARCHIVE : ENVIRONEMENT VARIABLE
set +A tmpARCH $( find /d* -maxdepth 1 -type d -name $LOGIN )
typeset -r ARCHIVE=${ARCHIVE:=${tmpARCH[0]}}
# if [ -z ${ARCHIVE} ] ; then
#     echo "   ARCHIVE variable was not set on claude."
#     echo "On this computer, you must set :"
#     echo "export ARCHIVE=/d1"
#     echo "(syntaxe depends of your shell)."
#     echo "Then your IGCM_OUT dir will be on \"/d1/${LOGIN}/\" ."
#     exit 1
# fi

#====================================================
#- IN
typeset -r R_IN=${R_IN:=/sanipsl/IGCM}

#====================================================
#- OUT
typeset -r R_OUT=${ARCHIVE}/IGCM_OUT

#====================================================
#- OUT_POST
typeset -r R_OUT_POST=${R_OUT}

#====================================================
#- RUN_DIR_PATH : Temporary working directory (=> TMP)
typeset -r RUN_DIR_PATH=${RUN_DIR_PATH:=${SCRATCHDIR}/tmp$$}

#=========================================================
#- Add "time" before mpirun command
MPIRUN_COMMAND="time "${MPIRUN_COMMAND}
echo ${MPIRUN_COMMAND}

#D-#==================================================
#D-function IGCM_sys_RshMaster
#D-* Purpose: Master rsh command
#D-* Examples:
#D-
function IGCM_sys_RshMaster {
    IGCM_debug_PushStack "IGCM_sys_RshMaster" $@
    ssh ${HOST} /bin/ksh  <<-EOF
    export libIGCM=${libIGCM}
    . ${libIGCM}/libIGCM_debug/libIGCM_debug.ksh
    . ${libIGCM}/libIGCM_card/libIGCM_card.ksh
    ${@}
EOF
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_RshMaster : erreur."
	IGCM_debug_Exit "IGCM_sys_RshMaster"
    fi
    IGCM_debug_PopStack "IGCM_sys_RshMaster"
}

#D-#==================================================
#D-function IGCM_sys_RshArchive
#D-* Purpose: Archive rsh command
#D-* Examples:
#D-
function IGCM_sys_RshArchive {
    IGCM_debug_PushStack "IGCM_sys_RshArchive" $@
    /bin/ksh  <<-EOF
    ${@}
EOF
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_RshArchive : erreur."
	IGCM_debug_Exit "IGCM_sys_RshArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_RshArchive"
}

#D-#==================================================
#D-function IGCM_sys_Nb_files_archive
#D-* Purpose: Count files on Archive filesystem
#D-* Examples:
#D-
function IGCM_sys_Nb_files_archive {
   ls ${@} 2>/dev/null | wc -l
   if [ $? -gt 0 ] ; then
       echo "IGCM_sys_RshArchive : erreur."
   fi
}

#D-#==================================================
#D-function IGCM_sys_RshPost
#D-* Purpose: Master rsh command
#D-* Examples:
#D-
(( RshPNum = 0 ))
function IGCM_sys_RshPost {
    IGCM_debug_PushStack "IGCM_sys_RshPost" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_RshPost :" $@
    fi
    #ssh ${HOST} exec /bin/ksh -vx $@ > out_RshPost.${RshPNum}
    ssh ${HOST} exec /bin/ksh $@ > out_RshPost.${RshPNum}
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_RshPost : erreur."
	IGCM_debug_Exit "IGCM_sys_RshPost"
    fi
    (( RshPNum = RshPNum + 1 ))
    IGCM_debug_PopStack "IGCM_sys_RshPost"
}

#D-#==================================================
#D-function IGCM_sys_Mkdir
#D-* Purpose: Master locale mkdir command
#D-* Examples:
#D-
function IGCM_sys_Mkdir {
    IGCM_debug_PushStack "IGCM_sys_Mkdir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Mkdir :" $@
    fi
    if [ ! -d ${1} ]; then
	\mkdir -p $1
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_Mkdir : erreur."
	    IGCM_debug_Exit "IGCM_sys_Mkdir"
	fi
    fi
    # vrification :
    if [ ! -d ${1} ] ; then
	echo "IGCM_sys_Mkdir : erreur."
	IGCM_debug_Exit "IGCM_sys_Mkdir"
    fi
    IGCM_debug_PopStack "IGCM_sys_Mkdir"
}

#D-#==================================================
#D-function IGCM_sys_MkdirArchive
#D-* Purpose: Mkdir on Archive
#D-* Examples:
#D-
function IGCM_sys_MkdirArchive {
    IGCM_debug_PushStack "IGCM_sys_MkdirArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_MkdirArchive :" $@
    fi
    #- creation de repertoire sur le serveur fichier
    if [ ! -d ${1} ]; then 
	\mkdir -p $1
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_MkdirArchive : erreur."
	    IGCM_debug_Exit "IGCM_sys_MkdirArchive"
	fi
    fi
    # vrification ?? :
    if [ ! -d ${1} ] ; then
	echo "IGCM_sys_MkdirArchive : erreur."
	IGCM_debug_Exit "IGCM_sys_MkdirArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_MkdirArchive"
}

#D-#==================================================
#D-function IGCM_sys_MkdirWork
#D-* Purpose: Mkdir on Work
#D-* Examples:
#D-
function IGCM_sys_MkdirWork {
    IGCM_debug_PushStack "IGCM_sys_MkdirWork" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_MkdirWork :" $@
    fi
    #- creation de repertoire sur le serveur fichier
    if [ ! -d ${1} ]; then 
	\mkdir -p $1
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_MkdirWork : erreur."
	    IGCM_debug_Exit "IGCM_sys_MkdirWork"
	fi
    fi
    # vrification ?? :
    if [ ! -d ${1} ] ; then
	echo "IGCM_sys_MkdirWork : erreur."
	IGCM_debug_Exit "IGCM_sys_MkdirWork"
    fi
    IGCM_debug_PopStack "IGCM_sys_MkdirWork"
}
IGCM_sys_MkdirWork ${RUN_DIR_PATH}
echo "RUN_DIR_PATH ${RUN_DIR_PATH} ok."

#D-#==================================================
#D-function IGCM_sys_Cd
#D-* Purpose: master cd command
#D-* Examples:
#D-
function IGCM_sys_Cd {
    IGCM_debug_PushStack "IGCM_sys_Cd" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Cd :" $@
    fi
    \cd $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Cd : erreur."
	IGCM_debug_Exit "IGCM_sys_Cd"
    fi
    IGCM_debug_PopStack "IGCM_sys_Cd"
}

#D-#==================================================
#D-function IGCM_sys_Chmod
#D-* Purpose: Chmod
#D-* Examples:
#D-
function IGCM_sys_Chmod {
    IGCM_debug_PushStack "IGCM_sys_Chmod" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Chmod :" $@
    fi
    if [ $DRYRUN -le 1 ]; then
	\chmod $@
	if [ $? -gt 0 ] ; then
	    echo "IGCM_sys_Chmod : erreur."
	    IGCM_debug_Exit "IGCM_sys_Chmod"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Chmod"
}

#D-#==================================================
#D-function IGCM_sys_FileSize
#D-* Purpose: Filesize
#D-* Examples:
#D-
function IGCM_sys_FileSize {
    IGCM_debug_PushStack "IGCM_sys_FileSize" $@

    typeset sizeF
    set +A sizeF -- $( ls -la ${1} )
    if [ $? -gt 0 ] ; then
	IGCM_debug_Exit "IGCM_sys_FileSize"
    fi
    eval ${2}=${sizeF[4]}

    IGCM_debug_PopStack "IGCM_sys_FileSize"
}

#D-#==================================================
#D-function IGCM_sys_TestDir
#D-* Purpose: Test Directory that must exists
#D-* Examples:
#D-
function IGCM_sys_TestDir {
    IGCM_debug_PushStack "IGCM_sys_TestDir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestDir :" $@
    fi
    if [ ! -d ${1} ]; then
	echo "IGCM_sys_TestDir : Directory $1 does not exist."
	IGCM_debug_Exit "IGCM_sys_TestDir"
    fi
    IGCM_debug_PopStack "IGCM_sys_TestDir"
}

#D-#==================================================
#D-function IGCM_sys_TestDirArchive
#D-* Purpose: Test Directory that must exists on Archive
#D-* Examples:
#D-
function IGCM_sys_TestDirArchive {
    IGCM_debug_PushStack "IGCM_sys_TestDirArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestDirArchive :" $@
    fi
    if [ ! -d ${1} ]; then
	echo "IGCM_sys_TestDirArchive : Directory $1 does not exist on ${ARCHIVE}."
	IGCM_debug_Exit "IGCM_sys_TestDirArchive"
    fi
    IGCM_debug_PopStack "IGCM_sys_TestDirArchive"
}

#D-#==================================================
#D-function IGCM_sys_TestFileArchive
#D-* Purpose: Test file that must NOT EXISTS on Archive
#D-* Examples:
#D-
function IGCM_sys_TestFileArchive {
    IGCM_debug_PushStack "IGCM_sys_TestFileArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_TestFileArchive :" $@
    fi
    if [ ! -f ${1} ]; then
	IGCM_debug_PopStack "IGCM_sys_TestFileArchive"
	return 1
    else
	IGCM_debug_PopStack "IGCM_sys_TestFileArchive"
	return 0
    fi
}

#D-#==================================================
#D-function IGCM_sys_Tree
#D-* Purpose: Tree directories with files on ${ARCHIVE}
#D-* Examples: IGCM_sys_Tree ${R_IN} ${R_OUT}
#D-
function IGCM_sys_Tree {
    IGCM_debug_PushStack "IGCM_sys_Tree" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Tree :" $@
    fi

    \tree -f $@

    IGCM_debug_PopStack "IGCM_sys_Tree"
}

#D-#==================================================
#D-function IGCM_sys_Tar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_Tar {
    IGCM_debug_PushStack "IGCM_sys_Tar" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Tar :" $@
    fi
    \tar cvf $@
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Tar : erreur."
	IGCM_debug_Exit "IGCM_sys_Tar"
    fi
    \tar tvf $1

    IGCM_debug_PopStack "IGCM_sys_Tar"
}

#D-#==================================================
#D-function IGCM_sys_UnTar
#D-* Purpose: master un-tar command
#D-* Examples:
#D-
function IGCM_sys_UnTar {
    IGCM_debug_PushStack "IGCM_sys_UnTar" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_UnTar :" $@
    fi
    \tar xvf $1
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_UnTar : erreur."
	IGCM_debug_Exit "IGCM_sys_UnTar"
    fi
    IGCM_debug_PopStack "IGCM_sys_UnTar"
}

#D-#==================================================
#D-function IGCM_sys_Qsub
#D-* Purpose: Qsub new job
#D-* Examples:
#D-
function IGCM_sys_Qsub {
    IGCM_debug_PushStack "IGCM_sys_Qsub" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Qsub :" $@
    fi
    echo "ssh -l ${LOGIN} ${HOST} ""cd ${SUBMIT_DIR} ; ${@}"" > ${Script_Output} 2>&1 &"
    ssh -l ${LOGIN} ${HOST} "cd ${SUBMIT_DIR} ; ${@}" > ${Script_Output} 2>&1 &
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_Qsub : erreur."
	IGCM_debug_Exit "IGCM_sys_Qsub"
    fi
    IGCM_debug_PopStack "IGCM_sys_Qsub"
}

#D-#==================================================
#D-function IGCM_sys_QsubPost
#D-* Purpose: Qsub new job on scalaire
#D-* Examples:
#D-
function IGCM_sys_QsubPost {
    IGCM_debug_PushStack "IGCM_sys_QsubPost" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_QsubPost :" $@
    fi
    #echo "ssh -x -l ${LOGIN} ${HOST} ${3} > ${2} 2>&1 &"
    #    ssh -x -l ${LOGIN} ${HOST} 
    echo "${3} > ${2} 2>&1 &"
    ${3} > ${2} 2>&1 &
    if [ $? -gt 0 ] ; then
	echo "IGCM_sys_QsubPost : erreur."
	IGCM_debug_Exit "IGCM_sys_QsubPost"
    fi
    IGCM_debug_PopStack "IGCM_sys_QsubPost"
}

#D-*************************
#D- File transfer functions 
#D-*************************
#D-

#D-#==================================================
#D-function IGCM_sys_Rsync_out
#D-* Purpose: treat return val of rsync
#D-* Examples:  IGCM_sys_Rsync_out out_RET_rsync
#D-  Error values and explanations can depend on your system version.
function IGCM_sys_Rsync_out {
    RET=$1
    if [ ! $RET ] ; then
	echo "rsync error !"
    fi

    if [ $MYLANG = "fr" ]; then
	case $RET in
	    0)  return ;;
	    1)  echo "Erreur de rsync ; RERR_SYNTAX : "
		echo "Erreur de syntaxe ou d'utilisation."
		return;;
	    2)  echo "Erreur de rsync ; RERR_PROTOCOL : "
		echo "Incompatibilit de protocole."
		return;;
	    3)  echo "Erreur de rsync ; RERR_FILESELECT 3"
		echo "Erreurs  lors  de  la  slection des fichiers d'entre sortie et"
		echo "rpertoires"
		return;;
	    4)  echo "Erreur de rsync ; RERR_UNSUPPORTED"
		echo "Action demande non supporte : une tentative de manipulation de"
		echo "fichiers  64-bits  sur une plate-forme qui ne les supporte pas a"
		echo "t faite ; ou une option qui est supporte par le  client  mais"
		echo "pas par le serveur a t spcifie."
		return;;
	    10) echo "Erreur de rsync ; RERR_SOCKETIO"
		echo "Erreur dans le socket d'entre sortie"
		return;;
	    11) echo "Erreur de rsync ; RERR_FILEIO"
		echo "Erreur d'entre sortie fichier"
		return;;
	    12) echo "Erreur de rsync ; RERR_STREAMIO"
		echo "Erreur dans flux de donne du protocole rsync"
		return;;
	    13) echo "Erreur de rsync ; RERR_MESSAGEIO"
		echo "Erreur avec les diagnostics du programme"
		return;;
	    14) echo "Erreur de rsync ; RERR_IPC"
		echo "Erreur dans le code IPC"
		return;;
	    20) echo "Erreur de rsync ; RERR_SIGNAL"
		echo "SIGUSR1 ou SIGINT reu"
		return;;
	    21) echo "Erreur de rsync ; RERR_WAITCHILD"
		echo "Une erreur retourne par waitpid()"
		return;;
	    22) echo "Erreur de rsync ; RERR_MALLOC"
		echo "Erreur lors de l'allocation des tampons de mmoire de coeur"
		return;;
	    23) echo ""
                echo "Erreur fichier inexistant"
		return;;
	    30) echo "Erreur de rsync ; RERR_TIMEOUT"
		echo "Temps d'attente coul dans l'envoi/rception de donnes"
		return;;
	    *)  echo "Erreur de rsync : code de retour de rsync inconnu :" $RET
		return;;
	esac
    elif [ $MYLANG = "en" ] ; then
	case $RET in
	    0)  return;;		
	    1)  echo "rsync error : Syntax or usage error "
		return;;
	    2)  echo "rsync error : Protocol incompatibility "
		return;;
	    3)  echo "rsync error : Errors selecting input/output files, dirs"
		return;;
	    4)  echo "rsync error : Requested action not supported: an attempt"
		echo "was made to manipulate 64-bit files on a platform that cannot support"
		echo "them; or an option was specified that is supported by the client and"
		echo "not by the server."
		return;;
	    5)  echo "rsync error : Error starting client-server protocol"
		return;;
	    10) echo "rsync error : Error in socket I/O "
		return;;
	    11) echo "rsync error : Error in file I/O "
		return;;
	    12) echo "rsync error : Error in rsync protocol data stream "
		return;;
	    13) echo "rsync error : Errors with program diagnostics "
		return;;
	    14) echo "rsync error : Error in IPC code "
		return;;
	    20) echo "rsync error : Received SIGUSR1 or SIGINT "
		return;;
	    21) echo "rsync error : Some error returned by waitpid() "
		return;;
	    22) echo "rsync error : Error allocating core memory buffers "
		return;;
	    23) echo "rsync error : Partial transfer due to error"
		return;;
	    24) echo "rsync error : Partial transfer due to vanished source files"
		return;;
	    30) echo "rsync error : Timeout in data send/receive "
		return;;
	    *)  echo "rsync error : return code of rsync unknown :" $RET
		return;;
	esac
    else
	echo "unknown language $MYLANG."
	return
    fi
}
    
#D-#==================================================
#D-function IGCM_sys_Cp
#D-* Purpose: generic cp
#D-* Examples:
#D-
function IGCM_sys_Cp {
    IGCM_debug_PushStack "IGCM_sys_Cp" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Cp :" $@
    fi

    typeset RET

    echo cp --preserve=timestamps $@ > out_rsync 2>&1
    \cp --preserve=timestamps $@ >> out_rsync 2>&1
    RET=$?
    
    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Cp : error."
	cat out_rsync
	IGCM_debug_Exit "IGCM_sys_Cp"
    fi
    IGCM_debug_PopStack "IGCM_sys_Cp"
}

#D-#==================================================
#D-function IGCM_sys_Rm
#D-* Purpose: generic rm
#D-* Examples:
#D-
function IGCM_sys_Rm {
    IGCM_debug_PushStack "IGCM_sys_Rm" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Rm :" $@
    fi

    typeset RET

    echo rm $@ > out_rsync 2>&1
    \rm $@ >> out_rsync 2>&1
    RET=$?
    
    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Rm : error."
	cat out_rsync
	IGCM_debug_Exit "IGCM_sys_Rm"
    fi
    IGCM_debug_PopStack "IGCM_sys_Rm"
}

#D-#==================================================
#D-function IGCM_sys_Mv
#D-* Purpose: generic move
#D-* Examples:
#D-
function IGCM_sys_Mv {
    IGCM_debug_PushStack "IGCM_sys_Mv" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Mv :" $@
    fi

    typeset RET

    echo mv $@ > out_rsync 2>&1
    \mv $@ >> out_rsync 2>&1
    RET=$?

    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Mv : error in mv."
	cat out_rsync
	IGCM_debug_Exit "IGCM_sys_Mv"
    fi

    IGCM_debug_PopStack "IGCM_sys_Mv"
}

#D-#==================================================
#D-function IGCM_sys_Put_Dir
#D-* Purpose: Copy a complete directory on $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Put_Dir {
    IGCM_debug_PushStack "IGCM_sys_Put_Dir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Dir :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -d ${1} ] ; then
	    echo "WARNING : IGCM_sys_Put_Dir ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Put_Dir"
	    return
	fi

	typeset RET

	# Only if we use rsync
	#IGCM_sys_TestDirArchive $( dirname $2 )
	#
	#USUAL WAY
	\cp -R $1 $2 > out_rsync 2>&1
  	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Dir : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Dir"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Get_Dir
#D-* Purpose: Copy a complete directory from $(ARCHIVE)
#D-* Examples:
#D-
function IGCM_sys_Get_Dir {
    IGCM_debug_PushStack "IGCM_sys_Get_Dir" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Get_Dir :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -d ${1} ] ; then
	    echo "WARNING : IGCM_sys_Get_Dir ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Get_Dir"
	    return
	fi

	typeset RET

	#USUAL WAY
	\cp -R $1 $2 > out_rsync 2>&1
  	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get_Dir : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Get_Dir"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Get_Dir"
}

#D-#==================================================
#D-function IGCM_sys_Get_Master
#D-* Purpose: Copy a complete directory from MASTER filesystem
#D-* Examples:
#D-
function IGCM_sys_Get_Master {
    IGCM_debug_PushStack "IGCM_sys_Get_Master" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Get_Master :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -d ${1} ] ; then
	    echo "WARNING : IGCM_sys_Get_Master ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Get_Master"
	    return
	fi

	typeset RET
	sleep 60

	#USUAL WAY
	cp -R $1 $2 > out_rsync 2>&1
  	RET=$?

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get_Master : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Get_Master"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Get_Master"
}

#D-#==================================================
#D-function IGCM_sys_Put_Rest
#D-* Purpose: commande de transfert des restarts $(ARCHIVE).
#D-           Quitte si le fichier ou si le rpertoire d'arriver n'existe pas.
#D-* Examples:
#D-
function IGCM_sys_Put_Rest {
    IGCM_debug_PushStack "IGCM_sys_Put_Rest" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Rest :" $@
    fi
    if [ $DRYRUN = 0 ]; then

	IGCM_sys_TestDirArchive $( dirname $2 )

	if [ ! -f ${1} ] ; then
	    echo "ERROR : IGCM_sys_Put_Rest ${1} DOES NOT EXIST ."
	    IGCM_debug_Exit "IGCM_sys_Put_Rest"
	fi
	IGCM_sys_Chmod 444 ${1}

	typeset RET

	echo ${RSYNC} ${RSYNC_opt} $1 $2 > out_rsync 2>&1
	${RSYNC} ${RSYNC_opt} $1 $2 >> out_rsync 2>&1
	RET=$?
	IGCM_sys_Rsync_out $RET

	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Rest : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Rest"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Rest"
}

#D-#==================================================
#D-function IGCM_sys_Put_Out
#D-* Purpose: Copy a file on $(ARCHIVE) after have chmod it in readonly
#D-* Examples:
#D-
function IGCM_sys_Put_Out {
    IGCM_debug_PushStack "IGCM_sys_Put_Out" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Out :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ -f ${1} ] ; then
	    if [ ! -d $( dirname $2 ) ] ; then
		IGCM_sys_MkdirArchive $( dirname $2 )
	    fi
	else
	    echo "WARNING : IGCM_sys_Put_Out ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Put_Out"
	    return 1
	fi

	typeset RET
	#
	IGCM_sys_Chmod 444 ${1}
	#

	echo ${RSYNC} ${RSYNC_opt} $1 $2 > out_rsync 2>&1
	${RSYNC} ${RSYNC_opt} $1 $2 >> out_rsync 2>&1
	RET=$?
	IGCM_sys_Rsync_out $RET

	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Put_Out : error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Put_Out"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Out"
    return 0
}

#D-#==================================================
#D-function IGCM_sys_Get
#D-* Purpose: Get a file from ${ARCHIVE}
#D-* Examples: IGCM_sys_Get myfile /destpath/myfile_with_PREFIX
#D-            IGCM_sys_Get /l Array_contain_myfiles /destpath/
function IGCM_sys_Get {
    IGCM_debug_PushStack "IGCM_sys_Get" $@

    typeset DEST RET dm_liste ifile target

    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Get :" $@
    fi
    if [ $DRYRUN -le 2 ]; then
	if [ X${1} = X'/l' ] ; then
            # test if the first file is present in the old computation :
	    eval set +A dm_liste \${${2}}
	else
	    dm_liste=${1}
	fi
	eval DEST=\${${#}}

	# test if the (first) file is present in the old computation :
	IGCM_sys_TestFileArchive ${dm_liste[0]}
	RET=$?
        if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get, ERROR : regular file ${dm_liste[0]} DOES NOT EXIST ."
	    IGCM_debug_Exit "IGCM_sys_Get"
	    return
	fi

	(( RET=0 ))
        for target in ${dm_liste[*]} ; do
	  ln -s ${target} ${DEST} >> out_rsync 2>&1
          (( RET=RET+$? ))
	done

#       echo ${RSYNC} ${RSYNC_opt} $@ > out_rsync 2>&1
#       ${RSYNC} ${RSYNC_opt} $@ >> out_rsync 2>&1
# 	RET=$?
# 	IGCM_sys_Rsync_out $RET

#  	${libIGCM}/libIGCM_sys/IGCM_analyse_rsync_out.awk out_rsync
#  	(( RET=RET+$? ))

	if [ ${RET} -gt 0 ] ; then
	    echo "IGCM_sys_Get : copy error."
	    cat out_rsync
	    IGCM_debug_Exit "IGCM_sys_Get"
	fi
    else
	( ${DEBUG_debug} ) && echo "DRYRUN mode = " $DRYRUN >> stack
    fi
    IGCM_debug_PopStack "IGCM_sys_Get"
}

#D-#==================================================
#D-function IGCM_sys_Put_Dods
#D-* Purpose: DO NOTHING ! Put $(ARCHIVE) files on DODS internet protocole.
#D-* Examples:
#D-
function IGCM_sys_Put_Dods {
    IGCM_debug_PushStack "IGCM_sys_Put_Dods" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Put_Dods :" $@
    fi
    if [ $DRYRUN = 0 ]; then
	if [ ! -f ${1} ] ; then
	    echo "WARNING : IGCM_sys_Put_Dods ${1} DOES NOT EXIST ."
	    IGCM_debug_PopStack "IGCM_sys_Put_Dods"
	    return
	fi
	#DO NOTHING
    else
	IGCM_err "DRYRUN mode = " $DRYRUN
    fi
    IGCM_debug_PopStack "IGCM_sys_Put_Dods"
}

############################################################## A REVOIR !!

#D-#==================================================
#D-function IGCM_sys_Rapatrie
#D-* Purpose: Rapatrie
#D-* Examples:
#D-
function IGCM_sys_Rapatrie {
    IGCM_debug_PushStack "IGCM_sys_Rapatrie" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_Rapatrie :" $@
    fi

    typeset RET=0

    IGCM_sys_Get ${R_STOCKAGE}/$2 $1 ; 
    let $(( RET=RET+$? ))
    IGCM_sys_Cd $1 ; 
    let $(( RET=RET+$? ))
    IGCM_sys_UnTar $2 ; 
    let $(( RET=RET+$? ))

    if [ ${RET} -gt 0 ] ; then
	echo "IGCM_sys_Rapatrie : erreur."
	IGCM_debug_Exit "IGCM_sys_Rapatrie"
    fi
    IGCM_debug_PopStack "IGCM_sys_Rapatrie"
}

############################################################## A FINIR !!

#D-#==================================================
#D-function IGCM_sys_GetDate_FichWork
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem WORK
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichWork {
    IGCM_debug_PushStack "IGCM_sys_FichWork" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_GetDate_FichWork :" $@
    fi
    # donne la date filesys d'un fichier sur la machine work
    IGCM_debug_PopStack "IGCM_sys_FichWork"
}

#D-#==================================================
#D-function IGCM_sys_GetDate_FichArchive
#D-* Purpose: donne la date filesys d'un fichier sur le filesystem ARCHIVE 
#D-* Examples:
#D-
function IGCM_sys_GetDate_FichArchive {
    IGCM_debug_PushStack "IGCM_sys_FichArchive" $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_GetDate_FichArchive :" $@
    fi
    IGCM_debug_PopStack "IGCM_sys_FichArchive"
}


##############################################################
# NCO OPERATOR

function IGCM_sys_ncap {
    IGCM_debug_PushStack "IGCM_sys_ncap" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncap :" $@
    fi
    /usr/bin/ncap $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncap : erreur ${@}."
#       IGCM_debug_Exit "ncap"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncap"
}

ncatted=/usr/bin/ncatted
# Problem with ksh and string passed in this function to ncatted !
# function IGCM_sys_ncatted {
#     IGCM_debug_PushStack "IGCM_sys_ncatted" -- $@
#     if ( $DEBUG_sys ) ; then
# 	echo "IGCM_sys_ncatted :" $@
#     fi
#     /usr/bin/ncatted $@
#     if [ $? -gt 0 ] ; then
#        echo "IGCM_sys_ncatted : erreur ${@}."
#        IGCM_debug_Exit "ncatted"
#     fi

#     IGCM_debug_PopStack "IGCM_sys_ncatted"
# }

function IGCM_sys_ncbo {
    IGCM_debug_PushStack "IGCM_sys_ncbo" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncbo :" $@
    fi
    /usr/bin/ncbo $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncbo : erreur ${@}."
#       IGCM_debug_Exit "ncbo"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncbo"
}

function IGCM_sys_ncdiff {
    IGCM_debug_PushStack "IGCM_sys_ncdiff" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncdiff :" $@
    fi
    /usr/bin/ncdiff $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncdiff : erreur ${@}."
#       IGCM_debug_Exit "ncdiff"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncdiff"
}

function IGCM_sys_ncea {
    IGCM_debug_PushStack "IGCM_sys_ncea" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncea :" $@
    fi
    /usr/bin/ncea $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncea : erreur ${@}."
#       IGCM_debug_Exit "ncea"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncea"
}

function IGCM_sys_ncecat {
    IGCM_debug_PushStack "IGCM_sys_ncecat" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncecat :" $@
    fi
    /usr/bin/ncecat $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncecat : erreur ${@}."
#       IGCM_debug_Exit "ncecat"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncecat"
}

function IGCM_sys_ncflint {
    IGCM_debug_PushStack "IGCM_sys_ncflint" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncflint :" $@
    fi
    /usr/bin/ncflint $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncflint : erreur ${@}."
#       IGCM_debug_Exit "ncflint"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncflint"
}

function IGCM_sys_ncks {
    IGCM_debug_PushStack "IGCM_sys_ncks" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncks :" $@
    fi
    /usr/bin/ncks $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncks : erreur ${@}."
#       IGCM_debug_Exit "ncks"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncks"
}

function IGCM_sys_ncpdq {
    IGCM_debug_PushStack "IGCM_sys_ncpdq" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncpdq :" $@
    fi
    /usr/bin/ncpdq $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncpdq : erreur ${@}."
#       IGCM_debug_Exit "ncpdq"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncpdq"
}

function IGCM_sys_ncra {
    IGCM_debug_PushStack "IGCM_sys_ncra" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncra :" $@
    fi
    /usr/bin/ncra $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncra : erreur ${@}."
#       IGCM_debug_Exit "ncra"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncra"
}

function IGCM_sys_ncrcat {
    IGCM_debug_PushStack "IGCM_sys_ncrcat" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncrcat :" $@
    fi
    /usr/bin/ncrcat $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncrcat : erreur ${@}."
#       IGCM_debug_Exit "ncrcat"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncrcat"
}

function IGCM_sys_ncrename {
    IGCM_debug_PushStack "IGCM_sys_ncrename" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncrename :" $@
    fi
    /usr/bin/ncrename $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncrename : erreur ${@}."
#       IGCM_debug_Exit "ncrename"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncrename"
}

function IGCM_sys_ncwa {
    IGCM_debug_PushStack "IGCM_sys_ncwa" -- $@
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_ncwa :" $@
    fi
    /usr/bin/ncwa $@
    if [ $? -gt 0 ] ; then
       echo "IGCM_sys_ncwa : erreur ${@}."
#       IGCM_debug_Exit "ncwa"
    fi

    IGCM_debug_PopStack "IGCM_sys_ncwa"
}

############################################################
# Activate Running Environnment Variables

function IGCM_sys_activ_variables {
    IGCM_debug_PushStack "IGCM_sys_activ_variables"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_activ_variables"
    fi
    IGCM_debug_PopStack "IGCM_sys_activ_variables"
}

############################################################
# Desactivate Running Environnment Variables

function IGCM_sys_desactiv_variables {
    IGCM_debug_PushStack "IGCM_sys_desactiv_variables"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_desactiv_variables"
    fi
    IGCM_debug_PopStack "IGCM_sys_desactiv_variables"
}

############################################################
# Build run file

function IGCM_sys_build_run_file {
    IGCM_debug_PushStack "IGCM_sys_build_run_file"
    if ( $DEBUG_sys ) ; then
	echo "IGCM_sys_build_run_file"
    fi
    IGCM_debug_PopStack "IGCM_sys_build_run_file"
}
